package awais.instagrabber.utils.dialogs;

import android.Manifest;
import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.CompoundButton;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.AppCompatCheckBox;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.FragmentManager;

import com.google.android.material.bottomsheet.BottomSheetDialogFragment;

import awais.instagrabber.R;
import awais.instagrabber.activities.Login;
import awais.instagrabber.utils.DirectoryChooser;
import awais.instagrabber.utils.Utils;

import static awais.instagrabber.utils.dialogs.SettingConstants.AUTOLOAD_POSTS;
import static awais.instagrabber.utils.dialogs.SettingConstants.AUTOPLAY_VIDEOS;
import static awais.instagrabber.utils.dialogs.SettingConstants.BOTTOM_SNACK;
import static awais.instagrabber.utils.dialogs.SettingConstants.BOTTOM_TOOLBAR;
import static awais.instagrabber.utils.dialogs.SettingConstants.DOWNLOAD_USER_FOLDER;
import static awais.instagrabber.utils.dialogs.SettingConstants.FOLDER_PATH;
import static awais.instagrabber.utils.dialogs.SettingConstants.FOLDER_SAVE_TO;
import static awais.instagrabber.utils.dialogs.SettingConstants.MUTED_VIDEOS;
import static awais.instagrabber.utils.dialogs.SettingConstants.SHOW_FEED;

public class SettingsDialog extends BottomSheetDialogFragment {
    private Activity activity;
    private FragmentManager fragmentManager;
    private View btnSaveTo;
    private boolean somethingChanged = false;
    private final CompoundButton.OnCheckedChangeListener checkChangeListener = (checkBox, checked) -> {
        final SharedPreferences.Editor editor = Utils.sharedPreferences.edit();
        final int id = checkBox.getId();
        if (id == R.id.cbDownloadUsername) editor.putBoolean(DOWNLOAD_USER_FOLDER, checked);
        else if (id == R.id.cbBottomToolbar) editor.putBoolean(BOTTOM_TOOLBAR, checked);
        else if (id == R.id.cbBottomSnack) editor.putBoolean(BOTTOM_SNACK, checked);
        else if (id == R.id.cbAutoplayVideos) editor.putBoolean(AUTOPLAY_VIDEOS, checked);
        else if (id == R.id.cbMuteVideos) editor.putBoolean(MUTED_VIDEOS, checked);
        else if (id == R.id.cbAutoloadPosts) editor.putBoolean(AUTOLOAD_POSTS, checked);
        else if (id == R.id.cbShowFeed) editor.putBoolean(SHOW_FEED, checked);
        else if (id == R.id.cbSaveTo) {
            editor.putBoolean(FOLDER_SAVE_TO, checked);
            btnSaveTo.setEnabled(checked);
        }
        editor.apply();
        somethingChanged = true;
    };
    private final View.OnClickListener clickListener = v -> {
        if (v instanceof Button) {
            final int id = v.getId();

            if (id == R.id.btnLogin) {
                startActivity(new Intent(v.getContext(), Login.class));
                somethingChanged = true;
            } else if (id == R.id.btnTimeSettings) {
                new TimeSettingsDialog().show(fragmentManager, null);
            } else if (id == R.id.btnSaveTo) {
                if (ContextCompat.checkSelfPermission(activity, Manifest.permission.WRITE_EXTERNAL_STORAGE) == PackageManager.PERMISSION_DENIED)
                    requestPermissions(Utils.PERMS, 6200);
                else showDirectoryChooser();
            }
        } else ((ViewGroup) v).getChildAt(0).performClick();
    };

    @Override
    public void onRequestPermissionsResult(final int requestCode, @NonNull final String[] permissions, @NonNull final int[] grantResults) {
        if (requestCode != 6200) return;
        if (grantResults[0] == PackageManager.PERMISSION_GRANTED) showDirectoryChooser();
        else Toast.makeText(activity, R.string.direct_download_perms_ask, Toast.LENGTH_SHORT).show();
    }

    private void showDirectoryChooser() {
        FragmentManager fragmentManager = getFragmentManager();
        if (fragmentManager == null) fragmentManager = getChildFragmentManager();

        new DirectoryChooser().setInitialDirectory(Utils.sharedPreferences.getString(FOLDER_PATH, null))
                .setInteractionListener(path -> {
                    Utils.sharedPreferences.edit().putString(FOLDER_PATH, path).apply();
                    somethingChanged = true;
                }).show(fragmentManager, null);
    }

    @NonNull
    @Override
    public Dialog onCreateDialog(@Nullable final Bundle savedInstanceState) {
        final Dialog dialog = super.onCreateDialog(savedInstanceState);

        final Context context = getContext();
        activity = context instanceof Activity ? (Activity) context : getActivity();

        fragmentManager = getFragmentManager();
        if (fragmentManager == null) fragmentManager = getChildFragmentManager();

        final View contentView = View.inflate(activity, R.layout.main_settings_dialog, null);

        btnSaveTo = contentView.findViewById(R.id.btnSaveTo);
        contentView.findViewById(R.id.btnLogin).setOnClickListener(clickListener);
        contentView.findViewById(R.id.btnTimeSettings).setOnClickListener(clickListener);

        btnSaveTo.setOnClickListener(clickListener);

        final AppCompatCheckBox cbSaveTo = contentView.findViewById(R.id.cbSaveTo);
        final AppCompatCheckBox cbShowFeed = contentView.findViewById(R.id.cbShowFeed);
        final AppCompatCheckBox cbMuteVideos = contentView.findViewById(R.id.cbMuteVideos);
        final AppCompatCheckBox cbBottomSnack = contentView.findViewById(R.id.cbBottomSnack);
        final AppCompatCheckBox cbBottomToolbar = contentView.findViewById(R.id.cbBottomToolbar);
        final AppCompatCheckBox cbAutoloadPosts = contentView.findViewById(R.id.cbAutoloadPosts);
        final AppCompatCheckBox cbAutoplayVideos = contentView.findViewById(R.id.cbAutoplayVideos);
        final AppCompatCheckBox cbDownloadUsername = contentView.findViewById(R.id.cbDownloadUsername);

        cbSaveTo.setChecked(Utils.sharedPreferences.getBoolean(FOLDER_SAVE_TO, false));
        cbMuteVideos.setChecked(Utils.sharedPreferences.getBoolean(MUTED_VIDEOS, true));
        cbBottomSnack.setChecked(Utils.sharedPreferences.getBoolean(BOTTOM_SNACK, true));
        cbBottomToolbar.setChecked(Utils.sharedPreferences.getBoolean(BOTTOM_TOOLBAR, true));
        cbAutoplayVideos.setChecked(Utils.sharedPreferences.getBoolean(AUTOPLAY_VIDEOS, true));

        cbShowFeed.setChecked(Utils.sharedPreferences.getBoolean(SHOW_FEED, false));
        cbAutoloadPosts.setChecked(Utils.sharedPreferences.getBoolean(AUTOLOAD_POSTS, false));
        cbDownloadUsername.setChecked(Utils.sharedPreferences.getBoolean(DOWNLOAD_USER_FOLDER, false));

        setupListener(cbSaveTo);
        setupListener(cbShowFeed);
        setupListener(cbMuteVideos);
        setupListener(cbBottomSnack);
        setupListener(cbBottomToolbar);
        setupListener(cbAutoloadPosts);
        setupListener(cbAutoplayVideos);
        setupListener(cbDownloadUsername);

        btnSaveTo.setEnabled(cbSaveTo.isChecked());

        dialog.setContentView(contentView);

        return dialog;
    }

    @Override
    public void onDismiss(@NonNull final DialogInterface dialog) {
        super.onDismiss(dialog);
        if (somethingChanged && activity != null) activity.recreate();
    }

    private void setupListener(@NonNull final AppCompatCheckBox checkBox) {
        checkBox.setOnCheckedChangeListener(checkChangeListener);
        ((View) checkBox.getParent()).setOnClickListener(clickListener);
    }
}